@@ -204,148 +204,86 @@ class Config(object):
 		response["success"] = True
 		response["message"] = f"Successfully created a ssh config for client [{username}@{server}] (host alias: {username}.{server})."
 		return response
-	def check_keys(self, username=None, public_keys=[]):
-
-		# make readable.
-		if username == None: username = OWNER
-		sudo = OWNER != username
-		sudo_option = ""
-		if sudo: sudo_option = "sudo "
-		ssh_dir = Formats.FilePath(f"{HOME_BASE}/{username}/.ssh/", check_existance=False)
-		auth_keys = Formats.FilePath(f"{HOME_BASE}/{username}/.ssh/authorized_keys", check_existance=False)
-		ssh_dir.permission.set(permission=770, sudo=sudo, silent=True) # silent for when non existant.
-		auth_keys.permission.set(permission=770, sudo=sudo, silent=True) # silent for when non existant.
-
-		# get output.
-		command = []
-		if sudo: command += ["sudo"]
-		output = utils.__execute__(command + ["cat", f"{HOME_BASE}/{username}/.ssh/authorized_keys"], return_format="array")
-		
-		# handle ssh dir not installed.
-		try:
-			if "No such file or directory" in output[0]:
-				ssh_dir.check(
-					directory=True,
-					permission=770,
-					owner=username,
-					group="administrator",
-					sudo=sudo,)
-				auth_keys.check(
-					data="",
-					permission=770,
-					owner=username,
-					group="administrator",
-					sudo=sudo,)
-				output = []
-		except IndexError: a=1
+	def check_keys(self, username=None, public_keys=[], reversed=False):
 
 		# check if already present.
+		if username == None: username = OWNER
+		ssh_dir = Formats.FilePath(f"{HOME_BASE}/{username}/.ssh/", check_existance=False)
+		auth_keys = Formats.FilePath(f"{HOME_BASE}/{username}/.ssh/authorized_keys", check_existance=False)
+		output = self.__load_keys__(username)
 		for key in public_keys:
-			if key.replace("\n", "") not in output:
-				auth_keys.permission.set(permission=600, sudo=sudo)
-				ssh_dir.permission.set(permission=700, sudo=sudo)
-				key = key.replace("\n", "")
-				return r3sponse.error_response(f'Public key [{key}] is not activated.')
+			key = key.replace("\n", "")
+			if key not in [""]:
+				if not reversed and key not in output:
+					ssh_dir.permission.set(permission=700, sudo=True, silent=True) # silent for when non existant.
+					auth_keys.permission.set(permission=600, sudo=True, silent=True) # silent for when non existant.
+					auth_keys.ownership.set(owner=username, sudo=True)
+					ssh_dir.ownership.set(owner=username, sudo=True)
+					return r3sponse.error_response(f'Public key [{key}] is not activated.')
+				if reversed and key in output:
+					ssh_dir.permission.set(permission=700, sudo=True, silent=True) # silent for when non existant.
+					auth_keys.permission.set(permission=600, sudo=True, silent=True) # silent for when non existant.
+					auth_keys.ownership.set(owner=username, sudo=True)
+					ssh_dir.ownership.set(owner=username, sudo=True)
+					return r3sponse.error_response(f'Public key [{key}] is activated.')
 
 		# set correct permission.
-		auth_keys.permission.set(permission=600, sudo=sudo)
-		ssh_dir.permission.set(permission=700, sudo=sudo)
+		ssh_dir.permission.set(permission=700, sudo=True, silent=True) # silent for when non existant.
+		auth_keys.permission.set(permission=600, sudo=True, silent=True) # silent for when non existant.
+		auth_keys.ownership.set(owner=username, sudo=True)
+		ssh_dir.ownership.set(owner=username, sudo=True)
 
 		# success.
-		return r3sponse.success_response(f'Successfully confirmed that the specfied {len(public_keys)} public key(s) are activated.')
-
+		if not reversed:
+			return r3sponse.success_response(f'Successfully confirmed that the specfied {len(public_keys)} public key(s) are activated.')
+		else:
+			return r3sponse.success_response(f'Successfully confirmed that the specfied {len(public_keys)} public key(s) are not activated.')
 	def enable_keys(self, username=None, public_keys=[]):
 
-		# make readable.
-		if username == None: username = OWNER
-		sudo = OWNER != username
-		sudo_option = ""
-		if sudo: sudo_option = "sudo "
-		ssh_dir = Formats.FilePath(f"{HOME_BASE}/{username}/.ssh/", check_existance=False)
-		auth_keys = Formats.FilePath(f"{HOME_BASE}/{username}/.ssh/authorized_keys", check_existance=False)
-
-		# checks.
-		if not ssh_dir.exists(sudo=sudo):
-			ssh_dir.create(
-				directory=True,
-				permission=770,
-				owner=username,
-				group=None,
-				sudo=sudo,)
-		if not auth_keys.exists(sudo=sudo):
-			auth_keys.create(
-				directory=False,
-				data="",
-				permission=770,
-				owner=username,
-				group=None,
-				sudo=sudo,)
-
-		ssh_dir.permission.set(permission=770, sudo=sudo, silent=True) # silent for when non existant.
-		auth_keys.permission.set(permission=770, sudo=sudo, silent=True) # silent for when non existant.
-		auth_keys.ownership.set(owner=OWNER, sudo=sudo)
-		ssh_dir.ownership.set(owner=OWNER, sudo=sudo)
-
-		# get output.
-		command = []
-		if sudo: command = ["sudo"]
-		output = utils.__execute__(command + ["cat", f"{HOME_BASE}/{username}/.ssh/authorized_keys"], return_format="array")
-		print("AUTHORIZED KEYS:  ",output)
-		# handle ssh dir not installed.
-		try:
-			if "No such file or directory" in output[0]:
-				print('NO SUCH FILLEEELELELEELE')
-				ssh_dir.check(
-					directory=True,
-					permission=770,
-					owner=username,
-					group=None,
-					sudo=sudo,)
-				auth_keys.check(
-					data="",
-					permission=770,
-					owner=username,
-					group=None,
-					sudo=sudo,)
-				ssh_dir.permission.set(permission=770, sudo=sudo, silent=True) # silent for when non existant.
-				auth_keys.permission.set(permission=770, sudo=sudo, silent=True) # silent for when non existant.
-				auth_keys.ownership.set(owner=OWNER, sudo=sudo)
-				ssh_dir.ownership.set(owner=OWNER, sudo=sudo)
-				output = []
-		except IndexError: output = []
-
 		# check if already present.
+		if username == None: username = OWNER
+		output = self.__load_keys__(username)
+		new_keys = []
 		for key in public_keys:
-			if key.replace("\n", "") not in output:
-				key = key.replace('\n','')
-				os.system(f"{sudo_option}echo '{key}' >> {HOME_BASE}/{username}/.ssh/authorized_keys")
+			key = key.replace("\n", "")
+			if key not in [""]:
+				if key not in output:
+					output.append(key)
+		self.__save_keys__(username, output)
 
 		# check if added.
-		output = utils.__execute__(command + ["cat", f"{HOME_BASE}/{username}/.ssh/authorized_keys"], return_format="array")
-		for key in public_keys:
-			if key.replace("\n", "") not in output:
-				auth_keys.permission.set(permission=600, sudo=sudo)
-				ssh_dir.permission.set(permission=700, sudo=sudo)
-				auth_keys.ownership.set(owner=username, sudo=sudo)
-				ssh_dir.ownership.set(owner=username, sudo=sudo)
-				key = key.replace("\n", "")
-				return r3sponse.error_response(f'Failed to enable public key [{key}].')
+		response = self.check_keys(username, public_keys)
+		if response["error"] != None: return response
 	
-		# set correct permission.
-		auth_keys.permission.set(permission=600, sudo=sudo)
-		ssh_dir.permission.set(permission=700, sudo=sudo)
-		auth_keys.ownership.set(owner=username, sudo=sudo)
-		ssh_dir.ownership.set(owner=username, sudo=sudo)
-
 		# success.
 		return r3sponse.success_response(f'Successfully enabled {len(public_keys)} public key(s).')
 
 		#
 	def disable_keys(self, username=None, public_keys=[]):
 
+		# check if already present.
+		if username == None: username = OWNER
+		output = self.__load_keys__(username)
+		new_keys = []
+		for key in output:
+			key = key.replace("\n", "")
+			if key not in [""]:
+				if key not in public_keys: new_keys.append(key)
+		self.__save_keys__(username, new_keys)
+
+		# check if added.
+		response = self.check_keys(username, public_keys, reversed=True)
+		if response["error"] != None: return response
+	
+		# success.
+		return r3sponse.success_response(f'Successfully disabled {len(public_keys)} public key(s).')
+
+		#
+	def __load_keys__(self, username):
+
 		# make readable.
 		if username == None: username = OWNER
-		sudo = OWNER != username
+		sudo = OWNER != username or True
 		ssh_dir = Formats.FilePath(f"{HOME_BASE}/{username}/.ssh/", check_existance=False)
 		auth_keys = Formats.FilePath(f"{HOME_BASE}/{username}/.ssh/authorized_keys", check_existance=False)
 
@@ -371,63 +309,50 @@ class Config(object):
 		auth_keys.ownership.set(owner=OWNER, sudo=sudo)
 		ssh_dir.ownership.set(owner=OWNER, sudo=sudo)
 
-		# get output.
-		command = []
-		if sudo: command += ["sudo"]
+		if sudo: command = ["sudo"]
+		else: command = []
 		output = utils.__execute__(command + ["cat", f"{HOME_BASE}/{username}/.ssh/authorized_keys"], return_format="array")
-		
-		# handle ssh dir not installed.
-		try:
-			if "No such file or directory" in output[0]:
-				ssh_dir.check(
-					directory=True,
-					permission=770,
-					owner=username,
-					group=None,
-					sudo=sudo,)
-				auth_keys.check(
-					data="",
-					permission=770,
-					owner=username,
-					group=None,
-					sudo=sudo,)
-				ssh_dir.permission.set(permission=770, sudo=sudo, silent=True) # silent for when non existant.
-				auth_keys.permission.set(permission=770, sudo=sudo, silent=True) # silent for when non existant.
-				auth_keys.ownership.set(owner=OWNER, sudo=sudo)
-				ssh_dir.ownership.set(owner=OWNER, sudo=sudo)
-				output = []
-		except IndexError: a=1
+		return output
 
-		# check if already present.
-		new_keys = ""
-		for key in output:
-			key = key.replace("\n", "")
-			if key.replace("\n", "") not in public_keys:
-				if new_keys == "":
-					new_keys = key+"\n"
-				else:
-					new_keys += key+"\n"
-		utils.__save_file__(f"{HOME_BASE}/{username}/.ssh/authorized_keys", new_keys)
+		#
+	def __save_keys__(self, username, public_keys):
 
-		# check if deleted.
-		output = utils.__execute__(command + ["cat", f"{HOME_BASE}/{username}/.ssh/authorized_keys"], return_format="array")
-		for key in public_keys:
-			key = key.replace("\n", "")
-			if key in output:
-				auth_keys.permission.set(permission=600, sudo=sudo)
-				ssh_dir.permission.set(permission=700, sudo=sudo)
-				auth_keys.ownership.set(owner=username, sudo=sudo)
-				ssh_dir.ownership.set(owner=username, sudo=sudo)
-				return r3sponse.error_response(f'Failed to disable public key [{key}].')
+		# make readable.
+		if username == None: username = OWNER
+		sudo = OWNER != username or True
+		ssh_dir = Formats.FilePath(f"{HOME_BASE}/{username}/.ssh/", check_existance=False)
+		auth_keys = Formats.FilePath(f"{HOME_BASE}/{username}/.ssh/authorized_keys", check_existance=False)
 
-		# set correct permission.
-		auth_keys.permission.set(permission=600, sudo=sudo)
-		ssh_dir.permission.set(permission=700, sudo=sudo)
-		auth_keys.ownership.set(owner=username, sudo=sudo)
-		ssh_dir.ownership.set(owner=username, sudo=sudo)
+		# checks.
+		if not ssh_dir.exists(sudo=sudo):
+			ssh_dir.create(
+				directory=True,
+				permission=770,
+				owner=username,
+				group=None,
+				sudo=sudo,)
+		if not auth_keys.exists(sudo=sudo):
+			auth_keys.create(
+				directory=False,
+				data="",
+				permission=770,
+				owner=username,
+				group=None,
+				sudo=sudo,)
 
-		# success.
-		return r3sponse.success_response(f'Successfully disabled {len(public_keys)} public key(s).')
+		f = Files.File(path="/tmp/file")
+		new = []
+		for public_key in public_keys:
+			new.append(public_key.replace("\n",''))
+		f.save(Files.Array(path=False, array=new).string(joiner="\n"))
+		os.system(f"sudo mv {f.file_path.path} {auth_keys.path}")
+
+		ssh_dir.permission.set(permission=700, sudo=sudo, silent=True) # silent for when non existant.
+		auth_keys.permission.set(permission=600, sudo=sudo, silent=True) # silent for when non existant.
+		auth_keys.ownership.set(owner=OWNER, sudo=sudo)
+		ssh_dir.ownership.set(owner=OWNER, sudo=sudo)
+
+		#
 
 # the ssh key object class.
 class Key(object):
